"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = __importDefault(require("../types"));
const iso_plugin_1 = require("./iso-plugin");
const webapp_plugin_1 = require("../webapp/webapp-plugin");
/**
 * The IsomorphicApp is an infrastructure and must implement [[IInfrastructure]]
 *
 * @param props
 */
exports.default = (props) => {
    console.log("isomorphic: ", props);
    const appProps = {
        // allows to identify this component as Infrastructure
        infrastructureType: types_1.default.INFRASTRUCTURE_TYPE_INFRASTRUCTURE,
        // only load plugins during compilation
        createPlugins: (configPath) => props.infrastructureMode === "COMPILATION" ? [
            // be able to process IsomorphicApps (as top-level-node)
            iso_plugin_1.IsoPlugin({
                buildPath: props.buildPath,
                configFilePath: configPath
            }),
            // isomorphic apps can have webapps (i.e. clients!)
            webapp_plugin_1.WebAppPlugin({
                buildPath: props.buildPath,
                configFilePath: configPath
            })
        ] : []
    };
    return Object.assign(appProps, props);
};
function isIsomorphicApp(component) {
    return component.props !== undefined &&
        component.props.stackName !== undefined &&
        component.props.buildPath !== undefined &&
        component.props.assetsPath !== undefined &&
        component.props.region !== undefined;
}
exports.isIsomorphicApp = isIsomorphicApp;
//# sourceMappingURL=iso-component.js.map