"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const iso_component_1 = require("./iso-component");
const sls_libs_1 = require("../utils/sls-libs");
const webpack_libs_1 = require("../utils/webpack-libs");
const iso_libs_1 = require("../utils/iso-libs");
const system_libs_1 = require("../utils/system-libs");
/**
 * A Plugin to detect Isomorphic-App-Components
 * @param props
 */
exports.IsoPlugin = (props) => {
    const result = {
        // identify Isomorphic-App-Components
        applies: (component) => {
            return iso_component_1.isIsomorphicApp(component);
        },
        // convert the component into configuration parts
        process: (component, childConfigs, infrastructureMode) => {
            const path = require('path');
            // we use the hardcoded name `server` as name
            const serverName = "server";
            // the isomorphic app has a server application
            const serverWebPack = webpack_libs_1.complementWebpackConfig(webpack_libs_1.createServerWebpackConfig("./" + path.join("node_modules", "infrastructure-scripts", "assets", "server.tsx"), //entryPath: string,
            path.join(system_libs_1.currentAbsolutePath(), props.buildPath), //use the buildpath from the parent plugin
            serverName, // name of the server
            {
                __CONFIG_FILE_PATH__: system_libs_1.pathToConfigFile(props.configFilePath) // replace the IsoConfig-Placeholder with the real path to the main-config-bundle
            }, {
                __ASSETS_PATH__: `"${component.props.assetsPath}"`,
                __RESOLVED_ASSETS_PATH__: `"${iso_libs_1.resolveAssetsPath(component.props.buildPath, serverName, component.props.assetsPath)}"`
            }));
            return {
                slsConfigs: [
                    sls_libs_1.toSlsConfig(component.props.stackName, serverName, component.props.buildPath, component.props.assetsPath, component.props.region),
                    ...childConfigs.map(config => config.slsConfigs)
                ],
                // provide all client configs and the server config in a flat list
                webpackConfigs: childConfigs.reduce((result, config) => result.concat(config.webpackConfigs), [serverWebPack]),
                postBuilds: childConfigs.reduce((result, config) => result.concat(config.postBuilds), []),
            };
        }
    };
    return result;
};
//# sourceMappingURL=iso-plugin.js.map