"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webapp_component_1 = require("./webapp-component");
const webpack_libs_1 = require("../utils/webpack-libs");
/**
 * A Plugin to detect WebApp-Components
 * @param props
 */
exports.MiddlewarePlugin = (props) => {
    const path = require('path');
    const result = {
        // identify Isomorphic-App-Components
        applies: (component) => {
            return webapp_component_1.isWebApp(component);
        },
        // convert the component into configuration parts
        process: (component, childConfigs, infrastructureMode) => {
            console.log("found webapp: ", component);
            // get the current path (at compilation time)
            const absolutePath = process.cwd().toString().replace(/(?:\r\n|\r|\n)/g, "");
            // where do we find the webpacked-configuration?
            const isoConfigPath = path.resolve(absolutePath, props.configFilePath.replace(/\.[^/.]+$/, ""));
            console.log("isoConfigPath: ", isoConfigPath);
            console.log("id: ", component.props.id);
            return {
                slsConfigs: [],
                // a webapp has its own webpack configuration
                webpackConfigs: [
                    webpack_libs_1.complementWebpackConfig(webpack_libs_1.createClientWebpackConfig("./" + path.join("node_modules", "infrastructure-scripts", "assets", "client.tsx"), //entryPath: string,
                    path.join(absolutePath, props.buildPath), //use the buildpath from the parent plugin
                    component.props.id, {
                        IsoConfig: isoConfigPath // replace the IsoConfig-Placeholder with the real path to the main-config-bundle
                    }, {
                        WEB_APP_ID: `"${component.props.id}"` // replace the webAppId-placeholder
                    }))
                ],
                postBuilds: [],
            };
        }
    };
    return result;
};
//# sourceMappingURL=middleware-plugin.js.map