"use strict";
/**
 * Starts an express-server on `localhost` that serves the client-web app on the port specified in
 * the environment variables or 3000 (default)
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const libs_1 = require("./libs");
const infrastructure_components_1 = require("infrastructure-components");
const sls_config_1 = require("./types/sls-config");
const ssr_config_1 = require("./types/ssr-config");
const iso_config_1 = require("./types/iso-config");
/**
 *
 * @param configFilePath
 */
function start(configFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield libs_1.loadConfiguration(configFilePath);
        // when we have a browser app, we start it directly
        if (config.type === infrastructure_components_1.ConfigTypes.LOWLEVEL_SPA && config.webpackConfig !== undefined) {
            console.log("start spa locally");
            libs_1.startDevServer(libs_1.complementWebpackConfig(config.webpackConfig));
        }
        else if (config.type === infrastructure_components_1.ConfigTypes.LOWLEVEL_SERVER && config.slsConfig !== undefined) {
            console.log("start llsrv locally/offline");
            sls_config_1.startSlsOffline(config.slsConfig, true);
        }
        else if (config.type === infrastructure_components_1.ConfigTypes.SSR && config.ssrConfig !== undefined) {
            console.log("start ssr locally/offline");
            ssr_config_1.startSsr(config.ssrConfig, config.slsConfig !== undefined ? config.slsConfig : {}, true);
        }
        else if (config.type === infrastructure_components_1.ConfigTypes.ISOMORPHIC && config.ssrConfig !== undefined && config.isoConfig !== undefined) {
            const { isoConfig, ssrConfig } = config;
            console.log("start iso locally/offline");
            ssr_config_1.startSsr((yield iso_config_1.isoToSsr(configFilePath, isoConfig, ssrConfig)), config.slsConfig !== undefined ? config.slsConfig : {}, true);
        } /* else if (config.type === ConfigTypes.ISOMORPHIC && config.ssrConfig !== undefined && config.isoConfig !== undefined) {
    
            // CURRENTLY: USE startSSr instead - works, too!
    
            console.log("start isomorphic locally/offline");
            startSsr(await isoToSsr(config.isoConfig, config.ssrConfig), true);
    
        } */
        else {
            console.error("Cannot start the provided configuration!");
        }
    });
}
exports.start = start;
//# sourceMappingURL=start.js.map