"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_lib_1 = require("./configuration-lib");
const config_parse_result_1 = require("./config-parse-result");
const infrastructure_1 = require("../types/infrastructure");
exports.INFRASTRUCTURE_MODES = {
    /**
     * When packing the application
     */
    COMPILATION: "COMPILATION",
    /**
     * when the app actually runs
     */
    RUNTIME: "RUNTIME"
};
/**
 * parses the configuration for plugins and returns a list of the plugins (objects)
 *
 * @param configPath the path to the compiled and evaluable configuration
 * @param origConfigPath the path to the original uncompiled configuration source!
 */
function parseForPlugins(configPath, origConfigPath) {
    const config = configuration_lib_1.loadConfiguration(configPath);
    const parsedComponent = exports.parseInfrastructureComponent(config, exports.INFRASTRUCTURE_MODES.COMPILATION);
    //console.log("configPath: ", configPath);
    if (infrastructure_1.isInfrastructure(parsedComponent)) {
        return infrastructure_1.extractPlugins(parsedComponent, origConfigPath);
    }
    else {
        console.error("main component is not a valid app!");
        return [];
    }
}
exports.parseForPlugins = parseForPlugins;
;
exports.parseInfrastructureComponent = (component, infrastructureMode) => {
    try {
        //console.log("parseInfrastructureComponent: ", component);
        const params = Object.assign({
            infrastructureMode: infrastructureMode,
        }, component.props);
        /*
        var custom = undefined;
        const parsed = `const f=${component.type}; f(${JSON.stringify(params)})`;

        const result = eval(parsed);*/
        const result = component.type(params);
        //
        //console.log("parsed: ", parsed);
        console.log("parsed InfrastructureComponent: ", result);
        return result.infrastructureType !== undefined ? result : undefined;
    }
    catch (error) {
        console.error("NOT an infrastructure component --> ", error);
        return undefined;
    }
};
/**
 * parses a configuration, this configuration must export the main component as default
 *
 *
 * @param component (main component of the configuration)
 * @param compileMode set to true to run the parser with a statically loaded configuration (without objects)
 */
function extractConfigs(component, plugins, infrastructureMode) {
    const results = plugins
        // check for plugins to apply
        .filter(plugin => plugin.applies(component))
        // apply applicable plugins
        .map(plugin => {
        const childConfigs = exports.getChildrenArray(component).map(child => extractConfigs(child, plugins, infrastructureMode));
        const r = plugin.process(component, childConfigs, infrastructureMode);
        console.log("result: ", r);
        return r;
    });
    return config_parse_result_1.mergeParseResults(results);
}
exports.extractConfigs = extractConfigs;
;
/**
 * Get the children of the current component as an array
 *
 *
 * @param component the parent component
 * @return an Array of the children, even if there is only a single child or no (empty array). If the component itself
 * is an array, its items are returned
 */
exports.getChildrenArray = (component) => {
    if (component == undefined) {
        return [];
    }
    if (Array.isArray(component) && component.length > 0) {
        //console.log("component is array: ", component)
        return [...component];
    }
    if (component.props == undefined || component.props.children == undefined) {
        return [];
    }
    return Array.isArray(component.props.children) ? component.props.children : [component.props.children];
};
//# sourceMappingURL=parser.js.map